@@ -31,12 +31,8 @@
 */
 
 #include <stdio.h>
-#include <stdlib.h>
-#include <stdint.h>
-#include <assert.h>
-#include <string.h>
-#include <unistd.h>
 #include <iostream>
+#include <sstream>
 
 #include "crypto.h"
 
@@ -56,36 +52,17 @@ int main( int argc, char *argv[] )
     Nonce nonce( myatoi( argv[ 1 ] ) );
 
     /* Read input */
-    char *input = NULL;
-    int total_size = 0;
-
-    while ( 1 ) {
-      unsigned char buf[ 16384 ];
-      ssize_t bytes_read = read( STDIN_FILENO, buf, 16384 );
-      if ( bytes_read == 0 ) { /* EOF */
-	break;
-      } else if ( bytes_read < 0 ) {
-	perror( "read" );
-	exit( 1 );
-      } else {
-	input = (char *)realloc( input, total_size + bytes_read );
-	assert( input );
-	memcpy( input + total_size, buf, bytes_read );
-	total_size += bytes_read;
-      }
-    }
-
-    string plaintext( input, total_size );
-    free( input );
+    ostringstream input;
+    input << cin.rdbuf();
 
     /* Encrypt message */
 
-    string ciphertext = session.encrypt( Message( nonce, plaintext ) );
+    string ciphertext = session.encrypt( Message( nonce, input.str() ) );
 
     cerr << "Key: " << key.printable_key() << endl;
 
     cout << ciphertext;
-  } catch ( CryptoException e ) {
+  } catch ( const CryptoException &e ) {
     cerr << e.text << endl;
     exit( 1 );
   }
