@@ -35,6 +35,7 @@
 #include <errno.h>
 #include <locale.h>
 #include <string.h>
+#include <sstream>
 #include <termios.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -51,7 +52,6 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <getopt.h>
 #include <time.h>
 #include <sys/stat.h>
 
@@ -81,6 +81,7 @@
 #include "pty_compat.h"
 #include "select.h"
 #include "timestamp.h"
+#include "fatal_assert.h"
 
 #ifndef _PATH_BSHELL
 #define _PATH_BSHELL "/bin/sh"
@@ -102,7 +103,7 @@ using namespace std;
 
 void print_usage( const char *argv0 )
 {
-  fprintf( stderr, "Usage: %s new [-s] [-v] [-i LOCALADDR] [-p PORT] [-c COLORS] [-l NAME=VALUE] [-- COMMAND...]\n", argv0 );
+  fprintf( stderr, "Usage: %s new [-s] [-v] [-i LOCALADDR] [-p PORT[:PORT2]] [-c COLORS] [-l NAME=VALUE] [-- COMMAND...]\n", argv0 );
 }
 
 void print_motd( void );
@@ -132,13 +133,10 @@ string get_SSH_IP( void )
     fprintf( stderr, "Warning: SSH_CONNECTION not found; binding to any interface.\n" );
     return string( "0.0.0.0" );
   }
-  char *SSH_writable = strdup( SSH_CONNECTION );
-  fatal_assert( SSH_writable );
-
-  strtok( SSH_writable, " " );
-  strtok( NULL, " " );
-  const char *local_interface_IP = strtok( NULL, " " );
-  if ( !local_interface_IP ) {
+  istringstream ss( SSH_CONNECTION );
+  string dummy, local_interface_IP;
+  ss >> dummy >> dummy >> local_interface_IP;
+  if ( !ss ) {
     fprintf( stderr, "Warning: Could not parse SSH_CONNECTION; binding to any interface.\n" );
     return string( "0.0.0.0" );
   }
@@ -146,12 +144,12 @@ string get_SSH_IP( void )
   /* Strip IPv6 prefix. */
   const char IPv6_prefix[] = "::ffff:";
 
-  if ( ( strlen( local_interface_IP ) > strlen( IPv6_prefix ) )
-       && ( 0 == strncasecmp( local_interface_IP, IPv6_prefix, strlen( IPv6_prefix ) ) ) ) {
-    return string( local_interface_IP + strlen( IPv6_prefix ) );
+  if ( ( local_interface_IP.length() > strlen( IPv6_prefix ) )
+       && ( 0 == strncasecmp( local_interface_IP.c_str(), IPv6_prefix, strlen( IPv6_prefix ) ) ) ) {
+    return local_interface_IP.substr( strlen( IPv6_prefix ) );
   }
 
-  return string( local_interface_IP );
+  return local_interface_IP;
 }
 
 int main( int argc, char *argv[] )
@@ -159,8 +157,12 @@ int main( int argc, char *argv[] )
   /* For security, make sure we don't dump core */
   Crypto::disable_dumping_core();
 
-  char *desired_ip = NULL;
-  char *desired_port = NULL;
+  /* Detect edge case */
+  fatal_assert( argc > 0 );
+
+  const char *desired_ip = NULL;
+  string desired_ip_str;
+  const char *desired_port = NULL;
   string command_path;
   char **command_argv = NULL;
   int colors = 0;
@@ -193,7 +195,8 @@ int main( int argc, char *argv[] )
 	desired_port = optarg;
 	break;
       case 's':
-	desired_ip = strdup( get_SSH_IP().c_str() );
+	desired_ip_str = get_SSH_IP();
+	desired_ip = desired_ip_str.c_str();
 	fatal_assert( desired_ip );
 	break;
       case 'c':
@@ -231,9 +234,9 @@ int main( int argc, char *argv[] )
     exit( 1 );
   }
 
-  if ( desired_port
-       && ( strspn( desired_port, "0123456789" ) != strlen( desired_port ) ) ) {
-    fprintf( stderr, "%s: Bad UDP port (%s)\n", argv[ 0 ], desired_port );
+  int dpl, dph;
+  if ( desired_port && ! Connection::parse_portrange( desired_port, dpl, dph ) ) {
+    fprintf( stderr, "%s: Bad UDP port range (%s)\n", argv[ 0 ], desired_port );
     print_usage( argv[ 0 ] );
     exit( 1 );
   }
@@ -242,6 +245,7 @@ int main( int argc, char *argv[] )
 
   /* Get shell */
   char *my_argv[ 2 ];
+  string shell_name;
   if ( !command_argv ) {
     /* get shell name */
     struct passwd *pw = getpwuid( geteuid() );
@@ -257,8 +261,6 @@ int main( int argc, char *argv[] )
 
     command_path = shell_path;
 
-    string shell_name;
-
     size_t shell_slash( shell_path.rfind('/') );
     if ( shell_slash == string::npos ) {
       shell_name = shell_path;
@@ -269,7 +271,7 @@ int main( int argc, char *argv[] )
     /* prepend '-' to make login shell */
     shell_name = '-' + shell_name;
 
-    my_argv[ 0 ] = strdup( shell_name.c_str() );
+    my_argv[ 0 ] = const_cast<char *>( shell_name.c_str() );
     my_argv[ 1 ] = NULL;
     command_argv = my_argv;
 
@@ -400,7 +402,7 @@ int run_server( const char *desired_ip, const char *desired_port,
 
     nullfd = open( "/dev/null", O_RDWR );
     if ( nullfd == -1 ) {
-      perror( "dup2" );
+      perror( "open" );
       exit( 1 );
     }
 
@@ -410,6 +412,11 @@ int run_server( const char *desired_ip, const char *desired_port,
       perror( "dup2" );
       exit( 1 );
     }
+
+    if ( close( nullfd ) < 0 ) {
+      perror( "close" );
+      exit( 1 );
+    }
   }
 
   char utmp_entry[ 64 ] = { 0 };
@@ -528,8 +535,9 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 
       const int timeout_if_no_client = 60000;
       int timeout = min( network.wait_time(), terminal.wait_time( now ) );
-      if ( !network.get_remote_state_num() ) {
-        timeout = min( timeout, timeout_if_no_client );
+      if ( (!network.get_remote_state_num())
+	   || network.shutdown_in_progress() ) {
+        timeout = min( timeout, 5000 );
       }
 
       /* poll for events */
@@ -538,7 +546,9 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
       assert( fd_list.size() == 1 ); /* servers don't hop */
       int network_fd = fd_list.back();
       sel.add_fd( network_fd );
-      sel.add_fd( host_fd );
+      if ( !network.shutdown_in_progress() ) {
+	sel.add_fd( host_fd );
+      }
 
       int active_fds = sel.select( timeout );
       if ( active_fds < 0 ) {
@@ -614,7 +624,7 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	}
       }
       
-      if ( sel.read( host_fd ) ) {
+      if ( (!network.shutdown_in_progress()) && sel.read( host_fd ) ) {
 	/* input from the host needs to be fed to the terminal */
 	const int buf_size = 16384;
 	char buf[ buf_size ];
@@ -625,24 +635,17 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
         /* If the pty slave is closed, reading from the master can fail with
            EIO (see #264).  So we treat errors on read() like EOF. */
         if ( bytes_read <= 0 ) {
-          bytes_read = 0;
-	  if ( !network.has_remote_addr() ) {
-	    spin(); /* let 60-second timer take care of this */
-	  } else if ( !network.shutdown_in_progress() ) {
-	    network.start_shutdown();
-	  }
-	}
+	  network.start_shutdown();
+	} else {
+	  string terminal_to_host = terminal.act( string( buf, bytes_read ) );
 	
-	string terminal_to_host = terminal.act( string( buf, bytes_read ) );
-	
-	/* update client with new state of terminal */
-	if ( !network.shutdown_in_progress() ) {
+	  /* update client with new state of terminal */
 	  network.set_current_state( terminal );
-	}
 
-	/* write any writeback octets back to the host */
-	if ( swrite( host_fd, terminal_to_host.c_str(), terminal_to_host.length() ) < 0 ) {
-	  break;
+	  /* write any writeback octets back to the host */
+	  if ( swrite( host_fd, terminal_to_host.c_str(), terminal_to_host.length() ) < 0 ) {
+	    break;
+	  }
 	}
       }
 
@@ -660,13 +663,9 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	break;
       }
 
-      if ( sel.error( host_fd ) ) {
+      if ( (!network.shutdown_in_progress()) && sel.error( host_fd ) ) {
 	/* host problem */
-	if ( network.has_remote_addr() ) {
-	  network.start_shutdown();
-	} else {
-	  spin(); /* let 60-second timer take care of this */
-	}
+	network.start_shutdown();
       }
 
       /* quit if our shutdown has been acknowledged */
@@ -685,9 +684,9 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
       }
 
       #ifdef HAVE_UTEMPTER
-      /* update utmp if has been more than 10 seconds since heard from client */
+      /* update utmp if has been more than 30 seconds since heard from client */
       if ( connected_utmp ) {
-	if ( time_since_remote_state > 10000 ) {
+	if ( time_since_remote_state > 30000 ) {
 	  utempter_remove_record( host_fd );
 
 	  char tmp[ 64 ];
@@ -776,6 +775,13 @@ bool motd_hushed( void )
   return (0 == lstat( ".hushlogin", &buf ));
 }
 
+bool device_exists( const char *ut_line )
+{
+  string device_name = string( "/dev/" ) + string( ut_line );
+  struct stat buf;
+  return (0 == lstat( device_name.c_str(), &buf ));
+}
+
 string mosh_read_line( FILE *file )
 {
   string ret;
@@ -810,8 +816,11 @@ void warn_unattached( const string & ignore_entry )
 	 && (username == string( entry->ut_user )) ) {
       /* does line show unattached mosh session */
       string text( entry->ut_host );
-      if ( (text.substr( 0, 5 ) == "mosh ")
-	   && (text != ignore_entry) ) {
+      if ( (text.size() >= 5)
+	   && (text.substr( 0, 5 ) == "mosh ")
+	   && (text[ text.size() - 1 ] == ']')
+	   && (text != ignore_entry)
+	   && device_exists( entry->ut_line ) ) {
 	unattached_mosh_servers.push_back( text );
       }
     }
