@@ -62,6 +62,21 @@
 
 #include "networktransport.cc"
 
+void STMClient::resume( void )
+{
+  /* Restore termios state */
+  if ( tcsetattr( STDIN_FILENO, TCSANOW, &raw_termios ) < 0 ) {
+      perror( "tcsetattr" );
+      exit( 1 );
+  }
+
+  /* Put terminal in application-cursor-key mode */
+  swrite( STDOUT_FILENO, display.open().c_str() );
+
+  /* Flag that outer terminal state is unknown */
+  repaint_requested = true;
+}
+
 void STMClient::init( void )
 {
   if ( !is_utf8_locale() ) {
@@ -99,7 +114,7 @@ void STMClient::init( void )
   }
 
   /* Put terminal in application-cursor-key mode */
-  swrite( STDOUT_FILENO, Terminal::Emulator::open().c_str() );
+  swrite( STDOUT_FILENO, display.open().c_str() );
 
   /* Add our name to window title */
   if ( !getenv( "MOSH_TITLE_NOPREFIX" ) ) {
@@ -120,7 +135,7 @@ void STMClient::shutdown( void )
   output_new_frame();
 
   /* Restore terminal and terminal-driver state */
-  swrite( STDOUT_FILENO, Terminal::Emulator::close().c_str() );
+  swrite( STDOUT_FILENO, display.close().c_str() );
   
   if ( tcsetattr( STDIN_FILENO, TCSANOW, &saved_termios ) < 0 ) {
     perror( "tcsetattr" );
@@ -146,7 +161,6 @@ void STMClient::main_init( void )
   sel.add_signal( SIGINT );
   sel.add_signal( SIGHUP );
   sel.add_signal( SIGPIPE );
-  sel.add_signal( SIGTSTP );
   sel.add_signal( SIGCONT );
 
   /* get initial window size */
@@ -241,6 +255,8 @@ bool STMClient::process_user_input( int fd )
 
       overlays.get_prediction_engine().new_user_byte( the_byte, *local_framebuffer );
 
+      const static wstring help_message( L"Commands: Ctrl-Z suspends, \".\" quits, \"^\" gives literal Ctrl-^" );
+
       if ( quit_sequence_started ) {
 	if ( the_byte == '.' ) { /* Quit sequence is Ctrl-^ . */
 	  if ( network->has_remote_addr() && (!network->shutdown_in_progress()) ) {
@@ -250,6 +266,23 @@ bool STMClient::process_user_input( int fd )
 	  } else {
 	    return false;
 	  }
+	} else if ( the_byte == 0x1a ) { /* Suspend sequence is Ctrl-^ Ctrl-Z */
+	  /* Restore terminal and terminal-driver state */
+	  swrite( STDOUT_FILENO, display.close().c_str() );
+
+	  if ( tcsetattr( STDIN_FILENO, TCSANOW, &saved_termios ) < 0 ) {
+	    perror( "tcsetattr" );
+	    exit( 1 );
+	  }
+
+	  printf( "\n\033[37;44m[mosh is suspended.]\033[m\n" );
+
+	  fflush( NULL );
+
+	  /* actually suspend */
+	  kill( 0, SIGSTOP );
+
+	  resume();
 	} else if ( the_byte == '^' ) {
 	  /* Emulation sequence to type Ctrl-^ is Ctrl-^ ^ */
 	  network->get_current_state().push_back( Parser::UserByte( 0x1E ) );
@@ -260,11 +293,17 @@ bool STMClient::process_user_input( int fd )
 	}
 
 	quit_sequence_started = false;
+
+	if ( overlays.get_notification_engine().get_notification_string() == help_message ) {
+	  overlays.get_notification_engine().set_notification_string( L"" );
+	}
+
 	continue;
       }
 
       quit_sequence_started = (the_byte == 0x1E);
       if ( quit_sequence_started ) {
+	overlays.get_notification_engine().set_notification_string( help_message, true, false );
 	continue;
       }
 
@@ -376,12 +415,14 @@ void STMClient::main( void )
         if ( !process_resize() ) { return; }
       }
 
+      if ( sel.signal( SIGCONT ) ) {
+	resume();
+      }
+
       if ( sel.signal( SIGTERM )
            || sel.signal( SIGINT )
            || sel.signal( SIGHUP )
-           || sel.signal( SIGPIPE )
-           || sel.signal( SIGTSTP )
-           || sel.signal( SIGCONT ) ) {
+           || sel.signal( SIGPIPE ) ) {
         /* shutdown signal */
         if ( !network->has_remote_addr() ) {
           break;
@@ -444,7 +485,7 @@ void STMClient::main( void )
       } else {
         overlays.get_notification_engine().clear_network_exception();
       }
-    } catch ( Network::NetworkException e ) {
+    } catch ( const Network::NetworkException &e ) {
       if ( !network->shutdown_in_progress() ) {
         overlays.get_notification_engine().set_network_exception( e );
       }
@@ -454,7 +495,7 @@ void STMClient::main( void )
       req.tv_nsec = 200000000; /* 0.2 sec */
       nanosleep( &req, NULL );
       freeze_timestamp();
-    } catch ( Crypto::CryptoException e ) {
+    } catch ( const Crypto::CryptoException &e ) {
       if ( e.fatal ) {
         throw;
       } else {
