@@ -168,9 +168,23 @@ NotificationEngine::NotificationEngine()
     last_acked_state( timestamp() ),
     message(),
     message_is_network_exception( false ),
-    message_expiration( -1 )
+    message_expiration( -1 ),
+    show_quit_keystroke( true )
 {}
 
+static std::string human_readable_duration( int num_seconds, const std::string &seconds_abbr ) {
+  char tmp[ 128 ];
+  if ( num_seconds < 60 ) {
+    snprintf( tmp, 128, "%d %s", num_seconds, seconds_abbr.c_str() );
+  } else if ( num_seconds < 3600 ) {
+    snprintf( tmp, 128, "%d:%02d", num_seconds / 60, num_seconds % 60 );
+  } else {
+    snprintf( tmp, 128, "%d:%02d:%02d", num_seconds / 3600,
+	      (num_seconds / 60) % 60, num_seconds % 60 );
+  }
+  return tmp;
+}
+
 void NotificationEngine::apply( Framebuffer &fb ) const
 {
   uint64_t now = timestamp();
@@ -219,15 +233,25 @@ void NotificationEngine::apply( Framebuffer &fb ) const
     explanation = reply_message;
   }
 
+  const static char quit_keystroke[] = " [To quit: Ctrl-^ .]";
+  const static char blank[] = "";
+
+  const char *keystroke_str = show_quit_keystroke ? quit_keystroke : blank;
+
   if ( message.empty() && (!time_expired) ) {
     return;
   } else if ( message.empty() && time_expired ) {
-    swprintf( tmp, 128, L"mosh: Last %s %.0f seconds ago. [To quit: Ctrl-^ .]", explanation, time_elapsed );
+    swprintf( tmp, 128, L"mosh: Last %s %s ago.%s", explanation,
+	      human_readable_duration( static_cast<int>( time_elapsed ),
+				       "seconds" ).c_str(),
+	      keystroke_str );
   } else if ( (!message.empty()) && (!time_expired) ) {
-    swprintf( tmp, 128, L"mosh: %ls [To quit: Ctrl-^ .]", message.c_str() );
+    swprintf( tmp, 128, L"mosh: %ls%s", message.c_str(), keystroke_str );
   } else {
-    swprintf( tmp, 128, L"mosh: %ls (%.0f s without %s.) [To quit: Ctrl-^ .]", message.c_str(),
-	      time_elapsed, explanation );
+    swprintf( tmp, 128, L"mosh: %ls (%s without %s.)%s", message.c_str(),
+	      human_readable_duration( static_cast<int>( time_elapsed ),
+				       "s" ).c_str(),
+	      explanation, keystroke_str );
   }
 
   wstring string_to_draw( tmp );
