@@ -101,22 +101,20 @@ namespace Network {
 
     static const int CONGESTION_TIMESTAMP_PENALTY = 500; /* ms */
 
-    static bool try_bind( int socket, uint32_t addr, int port );
+    static bool try_bind( int socket, uint32_t addr, int port_low, int port_high );
 
     class Socket
     {
     private:
       int _fd;
-      mutable bool _moved;
 
     public:
-      int fd( void ) const { assert( !_moved ); return _fd; }
-      void move( void ) const { assert( !_moved ); _moved = true; }
+      int fd( void ) const { return _fd; }
       Socket();
       ~Socket();
 
       Socket( const Socket & other );
-      const Socket & operator=( const Socket & other );
+      Socket & operator=( const Socket & other );
     };
 
     std::deque< Socket > socks;
@@ -185,6 +183,8 @@ namespace Network {
     }
 
     void set_last_roundtrip_success( uint64_t s_success ) { last_roundtrip_success = s_success; }
+
+    static bool parse_portrange( const char * desired_port_range, int & desired_port_low, int & desired_port_high );
   };
 }
 
