@@ -33,19 +33,63 @@
 /* This is in its own file because otherwise the ncurses #defines
    alias our own variable names. */
 
+#include "config.h"
 #include "terminaldisplay.h"
 
 #include <string>
 
-#include <curses.h>
-#include <term.h>
+#if defined HAVE_NCURSESW_CURSES_H
+#  include <ncursesw/curses.h>
+#  include <ncursesw/term.h>
+#elif defined HAVE_NCURSESW_H
+#  include <ncursesw.h>
+#  include <term.h>
+#elif defined HAVE_NCURSES_CURSES_H
+#  include <ncurses/curses.h>
+#  include <ncurses/term.h>
+#elif defined HAVE_NCURSES_H
+#  include <ncurses.h>
+#  include <term.h>
+#elif defined HAVE_CURSES_H
+#  include <curses.h>
+#  include <term.h>
+#else
+#  error "SysV or X/Open-compatible Curses header file required"
+#endif
 #include <stdlib.h>
 #include <string.h>
 
 using namespace Terminal;
 
+bool Display::ti_flag( const char *capname ) const
+{
+  int val = tigetflag( const_cast<char *>( capname ) );
+  if ( val == -1 ) {
+    throw std::string( "Invalid terminfo boolean capability " ) + capname;
+  }
+  return val;
+}
+
+int Display::ti_num( const char *capname ) const
+{
+  int val = tigetnum( const_cast<char *>( capname ) );
+  if ( val == -2 ) {
+    throw std::string( "Invalid terminfo numeric capability " ) + capname;
+  }
+  return val;
+}
+
+const char *Display::ti_str( const char *capname ) const
+{
+  const char *val = tigetstr( const_cast<char *>( capname ) );
+  if ( val == (const char *)-1 ) {
+    throw std::string( "Invalid terminfo string capability " ) + capname;
+  }
+  return val;
+}
+
 Display::Display( bool use_environment )
-  : has_ech( true ), has_bce( true ), has_title( true ), posterize_colors( false )
+  : has_ech( true ), has_bce( true ), has_title( true ), posterize_colors( false ), smcup( NULL ), rmcup( NULL )
 {
   if ( use_environment ) {
     int errret = -2;
@@ -69,22 +113,10 @@ Display::Display( bool use_environment )
     }
 
     /* check for ECH */
-    char ech_name[] = "ech";
-    char *val = tigetstr( ech_name );
-    if ( val == (char *)-1 ) {
-      throw std::string( "Invalid terminfo string capability " ) + ech_name;
-    } else if ( val == 0 ) {
-      has_ech = false;
-    }
+    has_ech = ti_str( "ech" );
 
     /* check for BCE */
-    char bce_name[] = "bce";
-    int bce_val = tigetflag( bce_name );
-    if ( bce_val == -1 ) {
-      throw std::string( "Invalid terminfo boolean capability " ) + bce_name;
-    } else if ( bce_val == 0 ) {
-      has_bce = false;
-    }
+    has_bce = ti_flag( "bce" );
 
     /* Check if we can set the window title and icon name.  terminfo does not
        have reliable information on this, so we hardcode a whitelist of
@@ -111,13 +143,12 @@ Display::Display( bool use_environment )
     /* posterization disabled because server now only advertises
        xterm-256color when client has colors = 256 */
     /*
-    char colors_name[] = "colors";
-    int color_val = tigetnum( colors_name );
-    if ( color_val == -2 ) {
-      throw std::string( "Invalid terminfo numeric capability " ) + colors_name;
-    } else if ( color_val < 256 ) {
-      posterize_colors = true;
-    }
+    posterize_colors = ti_num( "colors" ) < 256;
     */
+
+    if ( !getenv( "MOSH_NO_TERM_INIT" ) ) {
+      smcup = ti_str("smcup");
+      rmcup = ti_str("rmcup");
+    }
   }
 }
